# Maintaining an Openappstack cluster

## Logging

Logs from pods and containers can be read in different ways:

* In the cluster filesystem at `/var/log/pods/` or `/var/logs/containers/`.
* Using [kubectl logs](https://kubectl.docs.kubernetes.io/pages/container_debugging/container_logs.html).
* Querying aggregated logs with grafana, see below.

## Central log aggregation

We use [promtail](https://github.com/grafana/loki/tree/master/docs/clients/promtail),
[Loki](https://grafana.com/oss/loki/) and [grafana](https://grafana.com/) for
easy access of aggregated logs.
The [Loki documentation](https://github.com/grafana/loki#documentation) is a
good starting point how this setup works, and the [Using Loki in Grafana](https://grafana.com/docs/grafana/latest/features/datasources/loki/)
gets you started with querying your cluster logs with grafana.

You will find the loki grafana integration on your cluster at https://grafana.example.org/explore
together with some generic query examples.

### LogQL query examples

Please also refer to the [LogQL documentation](https://github.com/grafana/loki/blob/master/docs/logql.md).

#### Flux

All messages from `flux`:

    {app="flux"}

`flux` messages containing `wordpress`:

    {app = "flux"} |= "wordpress"

`flux` messages containing `wordpress` without `unchanged` events (to only show
the installation messages):

    {app = "flux"} |= "wordpress" != "unchanged"

Filter out redundant `flux` messages:

    { app = "flux" } !~ "(unchanged | event=refreshed | method=Sync | component=checkpoint)"


#### Cert-manager

`cert-manager` messages containing `chat`:

    {app="cert-manager"} |= "chat"


#### Hydra (single-sign-on system)

Show only warnings and errors from `hydra`:

    {container_name="hydra"} != "level=info"

## Backup

Please take care to backup the following locations:

### On your provisioning machine

* Your cluster config directory, located in the top-level sub-directory
  `clusters` in your clone of the openappstack git repository.
  Here you can find all the files generated during the `create` and `install`
  commands of the CLI, together with the generated secrets that are stored during
  installation.

### On your cluster

* The local storage directories under `/var/lib/OpenAppStack/local-storage`.
  This is the place all persistant volumes are stored.
  Some are more important than others, if you want to hand-pick what volumes
  to backup, use `kubectl get pvc --all-namespaces` to see which volumes are
  used by what application. The prometheus and alertmanager volume contain
  metrics, so you could choose to not back those up to save space.
* The [rke](https://rancher.com/docs/rke/latest/en/) directory `/var/lib/OpenAppStack/rke`
  where the `rke` config and state file of your cluster is stored.
* At this moment, [recurring, automated etcd snapshots are not configured](https://open.greenhost.net/openappstack/openappstack/issues/453).
  Please refer to the [rke etc snapshot
  documentation](https://rancher.com/docs/rke/latest/en/etcd-snapshots/) if you
  like to backup [etcd](https://etcd.io/).

If you don't care about your backup disk usage too much, the easiest way is to
backup the whole `/var/lib/OpenAppStack/` directory.

## Restore

Restore instructions will follow, please [reach out to us](https://openappstack.net/contact.html)
if you need assistance.
